@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
-#
-# $Id: _util.py 4218 2007-12-02 06:11:20Z piman $
 
 """Utility classes for Mutagen.
 
@@ -13,9 +11,75 @@ intended for internal use in Mutagen only.
 """
 
 import struct
+import codecs
 
 from fnmatch import fnmatchcase
 
+from ._compat import chr_, text_type, PY2, iteritems, iterbytes, integer_types
+
+
+class MutagenError(Exception):
+    """Base class for all custom exceptions in mutagen
+
+    .. versionadded:: 1.25
+    """
+
+
+def total_ordering(cls):
+    assert "__eq__" in cls.__dict__
+    assert "__lt__" in cls.__dict__
+
+    cls.__le__ = lambda self, other: self == other or self < other
+    cls.__gt__ = lambda self, other: not (self == other or self < other)
+    cls.__ge__ = lambda self, other: not self < other
+    cls.__ne__ = lambda self, other: not self.__eq__(other)
+
+    return cls
+
+
+def hashable(cls):
+    """Makes sure the class is hashable.
+
+    Needs a working __eq__ and __hash__ and will add a __ne__.
+    """
+
+    # py2
+    assert "__hash__" in cls.__dict__
+    # py3
+    assert cls.__dict__["__hash__"] is not None
+    assert "__eq__" in cls.__dict__
+
+    cls.__ne__ = lambda self, other: not self.__eq__(other)
+
+    return cls
+
+
+def enum(cls):
+    assert cls.__bases__ == (object,)
+
+    d = dict(cls.__dict__)
+    new_type = type(cls.__name__, (int,), d)
+    new_type.__module__ = cls.__module__
+
+    map_ = {}
+    for key, value in iteritems(d):
+        if key.upper() == key and isinstance(value, integer_types):
+            value_instance = new_type(value)
+            setattr(new_type, key, value_instance)
+            map_[value] = key
+
+    def repr_(self):
+        if self in map_:
+            return "%s.%s" % (type(self).__name__, map_[self])
+        else:
+            return "%s(%s)" % (type(self).__name__, self)
+
+    setattr(new_type, "__repr__", repr_)
+
+    return new_type
+
+
+@total_ordering
 class DictMixin(object):
     """Implement the dict API using keys() and __*item__ methods.
 
@@ -34,73 +98,98 @@ class DictMixin(object):
     def __iter__(self):
         return iter(self.keys())
 
-    def has_key(self, key):
-        try: self[key]
-        except KeyError: return False
-        else: return True
-    __contains__ = has_key
+    def __has_key(self, key):
+        try:
+            self[key]
+        except KeyError:
+            return False
+        else:
+            return True
 
-    iterkeys = lambda self: iter(self.keys())
+    if PY2:
+        has_key = __has_key
+
+    __contains__ = __has_key
+
+    if PY2:
+        iterkeys = lambda self: iter(self.keys())
 
     def values(self):
-        return map(self.__getitem__, self.keys())
-    itervalues = lambda self: iter(self.values())
+        return [self[k] for k in self.keys()]
+
+    if PY2:
+        itervalues = lambda self: iter(self.values())
 
     def items(self):
-        return zip(self.keys(), self.values())
-    iteritems = lambda s: iter(s.items())
+        return list(zip(self.keys(), self.values()))
+
+    if PY2:
+        iteritems = lambda s: iter(s.items())
 
     def clear(self):
-        map(self.__delitem__, self.keys())
+        for key in list(self.keys()):
+            self.__delitem__(key)
 
     def pop(self, key, *args):
         if len(args) > 1:
             raise TypeError("pop takes at most two arguments")
-        try: value = self[key]
+        try:
+            value = self[key]
         except KeyError:
-            if args: return args[0]
-            else: raise
+            if args:
+                return args[0]
+            else:
+                raise
         del(self[key])
         return value
 
     def popitem(self):
-        try:
-            key = self.keys()[0]
-            return key, self.pop(key)
-        except IndexError: raise KeyError("dictionary is empty")
+        for key in self.keys():
+            break
+        else:
+            raise KeyError("dictionary is empty")
+        return key, self.pop(key)
 
     def update(self, other=None, **kwargs):
         if other is None:
             self.update(kwargs)
             other = {}
 
-        try: map(self.__setitem__, other.keys(), other.values())
+        try:
+            for key, value in other.items():
+                self.__setitem__(key, value)
         except AttributeError:
             for key, value in other:
                 self[key] = value
 
     def setdefault(self, key, default=None):
-        try: return self[key]
+        try:
+            return self[key]
         except KeyError:
             self[key] = default
             return default
 
     def get(self, key, default=None):
-        try: return self[key]
-        except KeyError: return default
+        try:
+            return self[key]
+        except KeyError:
+            return default
 
     def __repr__(self):
         return repr(dict(self.items()))
 
-    def __cmp__(self, other):
-        if other is None: return 1
-        else: return cmp(dict(self.items()), other)
+    def __eq__(self, other):
+        return dict(self.items()) == other
+
+    def __lt__(self, other):
+        return dict(self.items()) < other
 
     __hash__ = object.__hash__
 
     def __len__(self):
         return len(self.keys())
 
+
 class DictProxy(DictMixin):
     def __init__(self, *args, **kwargs):
         self.__dict = {}
@@ -118,10 +207,12 @@ class DictProxy(DictMixin):
     def keys(self):
         return self.__dict.keys()
 
+
 class cdata(object):
     """C character buffer to Python numeric type conversions."""
 
     from struct import error
+    error = error
 
     short_le = staticmethod(lambda data: struct.unpack('<h', data)[0])
     ushort_le = staticmethod(lambda data: struct.unpack('<H', data)[0])
@@ -159,13 +250,12 @@ class cdata(object):
     to_longlong_be = staticmethod(lambda data: struct.pack('>q', data))
     to_ulonglong_be = staticmethod(lambda data: struct.pack('>Q', data))
 
-    bitswap = ''.join([chr(sum([((val >> i) & 1) << (7-i) for i in range(8)]))
-                       for val in range(256)])
-    del(i)
-    del(val)
+    bitswap = b''.join(chr_(sum(((val >> i) & 1) << (7 - i) for i in range(8)))
+                       for val in range(256))
 
     test_bit = staticmethod(lambda value, n: bool((value >> n) & 1))
 
+
 def lock(fileobj):
     """Lock a file object 'safely'.
 
@@ -177,11 +267,14 @@ def lock(fileobj):
     raises an exception in more extreme circumstances (full
     lock table, invalid file).
     """
-    try: import fcntl
+
+    try:
+        import fcntl
     except ImportError:
         return False
     else:
-        try: fcntl.lockf(fileobj, fcntl.LOCK_EX)
+        try:
+            fcntl.lockf(fileobj, fcntl.LOCK_EX)
         except IOError:
             # FIXME: There's possibly a lot of complicated
             # logic that needs to go here in case the IOError
@@ -190,38 +283,44 @@ def lock(fileobj):
         else:
             return True
 
+
 def unlock(fileobj):
     """Unlock a file object.
 
     Don't call this on a file object unless a call to lock()
     returned true.
     """
+
     # If this fails there's a mismatched lock/unlock pair,
     # so we definitely don't want to ignore errors.
     import fcntl
     fcntl.lockf(fileobj, fcntl.LOCK_UN)
 
-def insert_bytes(fobj, size, offset, BUFFER_SIZE=2**16):
+
+def insert_bytes(fobj, size, offset, BUFFER_SIZE=2 ** 16):
     """Insert size bytes of empty space starting at offset.
 
     fobj must be an open file object, open rb+ or
     equivalent. Mutagen tries to use mmap to resize the file, but
     falls back to a significantly slower method if mmap fails.
     """
+
     assert 0 < size
     assert 0 <= offset
     locked = False
     fobj.seek(0, 2)
     filesize = fobj.tell()
     movesize = filesize - offset
-    fobj.write('\x00' * size)
+    fobj.write(b'\x00' * size)
     fobj.flush()
     try:
         try:
             import mmap
-            map = mmap.mmap(fobj.fileno(), filesize + size)
-            try: map.move(offset + size, offset, movesize)
-            finally: map.close()
+            file_map = mmap.mmap(fobj.fileno(), filesize + size)
+            try:
+                file_map.move(offset + size, offset, movesize)
+            finally:
+                file_map.close()
         except (ValueError, EnvironmentError, ImportError):
             # handle broken mmap scenarios
             locked = lock(fobj)
@@ -233,7 +332,7 @@ def insert_bytes(fobj, size, offset, BUFFER_SIZE=2**16):
             # the file out several megs.
             while padsize:
                 addsize = min(BUFFER_SIZE, padsize)
-                fobj.write("\x00" * addsize)
+                fobj.write(b"\x00" * addsize)
                 padsize -= addsize
 
             fobj.seek(filesize, 0)
@@ -259,13 +358,15 @@ def insert_bytes(fobj, size, offset, BUFFER_SIZE=2**16):
         if locked:
             unlock(fobj)
 
-def delete_bytes(fobj, size, offset, BUFFER_SIZE=2**16):
+
+def delete_bytes(fobj, size, offset, BUFFER_SIZE=2 ** 16):
     """Delete size bytes of empty space starting at offset.
 
     fobj must be an open file object, open rb+ or
     equivalent. Mutagen tries to use mmap to resize the file, but
     falls back to a significantly slower method if mmap fails.
     """
+
     locked = False
     assert 0 < size
     assert 0 <= offset
@@ -278,9 +379,11 @@ def delete_bytes(fobj, size, offset, BUFFER_SIZE=2**16):
             fobj.flush()
             try:
                 import mmap
-                map = mmap.mmap(fobj.fileno(), filesize)
-                try: map.move(offset, offset + size, movesize)
-                finally: map.close()
+                file_map = mmap.mmap(fobj.fileno(), filesize)
+                try:
+                    file_map.move(offset, offset + size, movesize)
+                finally:
+                    file_map.close()
             except (ValueError, EnvironmentError, ImportError):
                 # handle broken mmap scenarios
                 locked = lock(fobj)
@@ -298,19 +401,101 @@ def delete_bytes(fobj, size, offset, BUFFER_SIZE=2**16):
         if locked:
             unlock(fobj)
 
-def utf8(data):
-    """Convert a basestring to a valid UTF-8 str."""
-    if isinstance(data, str):
-        return data.decode("utf-8", "replace").encode("utf-8")
-    elif isinstance(data, unicode):
-        return data.encode("utf-8")
-    else: raise TypeError("only unicode/str types can be converted to UTF-8")
 
 def dict_match(d, key, default=None):
-    try:
+    """Like __getitem__ but works as if the keys() are all filename patterns.
+    Returns the value of any dict key that matches the passed key.
+    """
+
+    if key in d and "[" not in key:
         return d[key]
-    except KeyError:
-        for pattern, value in d.iteritems():
+    else:
+        for pattern, value in iteritems(d):
             if fnmatchcase(key, pattern):
                 return value
     return default
+
+
+def decode_terminated(data, encoding, strict=True):
+    """Returns the decoded data until the first NULL terminator
+    and all data after it.
+
+    In case the data can't be decoded raises UnicodeError.
+    In case the encoding is not found raises LookupError.
+    In case the data isn't null terminated (even if it is encoded correctly)
+    raises ValueError except if strict is False, then the decoded string
+    will be returned anyway.
+    """
+
+    codec_info = codecs.lookup(encoding)
+
+    # normalize encoding name so we can compare by name
+    encoding = codec_info.name
+
+    # fast path
+    if encoding in ("utf-8", "iso8859-1"):
+        index = data.find(b"\x00")
+        if index == -1:
+            # make sure we raise UnicodeError first, like in the slow path
+            res = data.decode(encoding), b""
+            if strict:
+                raise ValueError("not null terminated")
+            else:
+                return res
+        return data[:index].decode(encoding), data[index + 1:]
+
+    # slow path
+    decoder = codec_info.incrementaldecoder()
+    r = []
+    for i, b in enumerate(iterbytes(data)):
+        c = decoder.decode(b)
+        if c == u"\x00":
+            return u"".join(r), data[i + 1:]
+        r.append(c)
+    else:
+        # make sure the decoder is finished
+        r.append(decoder.decode(b"", True))
+        if strict:
+            raise ValueError("not null terminated")
+        return u"".join(r), b""
+
+
+def split_escape(string, sep, maxsplit=None, escape_char="\\"):
+    """Like unicode/str/bytes.split but allows for the separator to be escaped
+
+    If passed unicode/str/bytes will only return list of unicode/str/bytes.
+    """
+
+    assert len(sep) == 1
+    assert len(escape_char) == 1
+
+    if isinstance(string, bytes):
+        if isinstance(escape_char, text_type):
+            escape_char = escape_char.encode("ascii")
+        iter_ = iterbytes
+    else:
+        iter_ = iter
+
+    if maxsplit is None:
+        maxsplit = len(string)
+
+    empty = string[:0]
+    result = []
+    current = empty
+    escaped = False
+    for char in iter_(string):
+        if escaped:
+            if char != escape_char and char != sep:
+                current += escape_char
+            current += char
+            escaped = False
+        else:
+            if char == escape_char:
+                escaped = True
+            elif char == sep and len(result) < maxsplit:
+                result.append(current)
+                current = empty
+            else:
+                current += char
+    result.append(current)
+    return result
