@@ -3,8 +3,8 @@ from tempfile import mkstemp
 import shutil
 
 from tests import TestCase, add
-from mutagen._compat import cBytesIO, text_type
-from mutagen import File, Metadata, FileType
+from mutagen._compat import cBytesIO, PY3
+from mutagen import File, Metadata, FileType, MutagenError
 from mutagen.oggvorbis import OggVorbis
 from mutagen.oggflac import OggFLAC
 from mutagen.oggspeex import OggSpeex
@@ -21,13 +21,14 @@ from mutagen.monkeysaudio import MonkeysAudio
 from mutagen.optimfrog import OptimFROG
 from mutagen.asf import ASF
 from mutagen.aiff import AIFF
-try: from os.path import devnull
-except ImportError: devnull = "/dev/null"
+from os import devnull
+
 
 class TMetadata(TestCase):
 
     class FakeMeta(Metadata):
-        def __init__(self): pass
+        def __init__(self):
+            pass
 
     def test_virtual_constructor(self):
         self.failUnlessRaises(NotImplementedError, Metadata, "filename")
@@ -47,6 +48,7 @@ class TMetadata(TestCase):
             NotImplementedError, self.FakeMeta().delete, "filename")
 add(TMetadata)
 
+
 class TFileType(TestCase):
 
     def setUp(self):
@@ -64,10 +66,12 @@ class TFileType(TestCase):
         self.failIf("quux" in self.vorbis)
 add(TFileType)
 
+
 class TFile(TestCase):
 
     def test_bad(self):
-        try: self.failUnless(File(devnull) is None)
+        try:
+            self.failUnless(File(devnull) is None)
         except (OSError, IOError):
             print("WARNING: Unable to open %s." % devnull)
         self.failUnless(File(__file__) is None)
@@ -75,8 +79,10 @@ class TFile(TestCase):
     def test_empty(self):
         filename = os.path.join("tests", "data", "empty")
         open(filename, "wb").close()
-        try: self.failUnless(File(filename) is None)
-        finally: os.unlink(filename)
+        try:
+            self.failUnless(File(filename) is None)
+        finally:
+            os.unlink(filename)
 
     def test_not_file(self):
         self.failUnlessRaises(EnvironmentError, File, "/dev/doesnotexist")
@@ -202,22 +208,37 @@ class TFile(TestCase):
         self.failUnless(TrueAudio.score(filename, fileobj, header) <
                         MP3.score(filename, fileobj, header))
 
+    def test_prefer_theora_over_vorbis(self):
+        header = (
+            b"OggS\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00\xe1x\x06\x0f"
+            b"\x00\x00\x00\x00)S'\xf4\x01*\x80theora\x03\x02\x01\x006\x00\x1e"
+            b"\x00\x03V\x00\x01\xe0\x00\x00\x00\x00\x00\x18\x00\x00\x00\x01"
+            b"\x00\x00\x00\x00\x00\x00\x00&%\xa0\x00\xc0OggS\x00\x02\x00\x00"
+            b"\x00\x00\x00\x00\x00\x00d#\xa8\x1f\x00\x00\x00\x00]Y\xc0\xc0"
+            b"\x01\x1e\x01vorbis\x00\x00\x00\x00\x02\x80\xbb\x00\x00\x00\x00"
+            b"\x00\x00\x00\xee\x02\x00\x00\x00\x00\x00\xb8\x01")
+        fileobj = cBytesIO(header)
+        filename = "not-identifiable.ext"
+        self.failUnless(OggVorbis.score(filename, fileobj, header) <
+                        OggTheora.score(filename, fileobj, header))
+
 add(TFile)
 
+
 class TFileUpperExt(TestCase):
-    FILES = [(os.path.join(b"tests", b"data", b"empty.ofr"), OptimFROG),
-             (os.path.join(b"tests", b"data", b"sv5_header.mpc"), Musepack),
-             (os.path.join(b"tests", b"data", b"silence-3.wma"), ASF),
-             (os.path.join(b"tests", b"data", b"truncated-64bit.mp4"), MP4),
-             (os.path.join(b"tests", b"data", b"silence-44-s.flac"), FLAC),
-             ]
-             
+    FILES = [
+        (os.path.join("tests", "data", "empty.ofr"), OptimFROG),
+        (os.path.join("tests", "data", "sv5_header.mpc"), Musepack),
+        (os.path.join("tests", "data", "silence-3.wma"), ASF),
+        (os.path.join("tests", "data", "truncated-64bit.mp4"), MP4),
+        (os.path.join("tests", "data", "silence-44-s.flac"), FLAC),
+    ]
+
     def setUp(self):
         checks = []
         for (original, instance) in self.FILES:
-            ext = original.rsplit(b".", 1)[-1]
-            suffix = b'.' + ext.upper()
-            fd, filename = mkstemp(suffix=str(suffix))
+            ext = os.path.splitext(original)[1]
+            fd, filename = mkstemp(suffix=ext.upper())
             os.close(fd)
             shutil.copy(original, filename)
             checks.append((filename, instance))
@@ -242,15 +263,30 @@ add(TFileUpperExt)
 
 class TModuleImportAll(TestCase):
 
-    def test_all(self):
+    def setUp(self):
         import mutagen
         files = os.listdir(mutagen.__path__[0])
-        modules = [os.path.splitext(f)[0] for f in files]
+        modules = set(os.path.splitext(f)[0] for f in files)
         modules = [f for f in modules if not f.startswith("_")]
 
+        if PY3 and 'm4a' in modules:
+            modules.remove('m4a')
+
+        self.modules = []
         for module in modules:
             mod = getattr(__import__("mutagen." + module), module)
+            self.modules.append(mod)
+
+    def tearDown(self):
+        del self.modules[:]
+
+    def test_all(self):
+        for mod in self.modules:
             for attr in getattr(mod, "__all__", []):
                 getattr(mod, attr)
 
+    def test_errors(self):
+        for mod in self.modules:
+            self.assertTrue(issubclass(mod.error, MutagenError), msg=mod.error)
+
 add(TModuleImportAll)
