@@ -1,8 +1,15 @@
-from mutagen._util import DictMixin, cdata, utf8, insert_bytes, delete_bytes
-from mutagen._util import decode_terminated
+from mutagen._util import DictMixin, cdata, insert_bytes, delete_bytes
+from mutagen._util import decode_terminated, split_escape, dict_match, enum
 from mutagen._compat import text_type, itervalues, iterkeys, iteritems, PY2
 from tests import TestCase, add
 import random
+import mmap
+
+try:
+    import fcntl
+except ImportError:
+    fcntl = None
+
 
 class FDict(DictMixin):
 
@@ -10,37 +17,15 @@ class FDict(DictMixin):
         self.__d = {}
         self.keys = self.__d.keys
 
-    def __getitem__(self, *args): return self.__d.__getitem__(*args)
-    def __setitem__(self, *args): return self.__d.__setitem__(*args)
-    def __delitem__(self, *args): return self.__d.__delitem__(*args)
+    def __getitem__(self, *args):
+        return self.__d.__getitem__(*args)
 
-class Tutf8(TestCase):
+    def __setitem__(self, *args):
+        return self.__d.__setitem__(*args)
 
-    def test_str(self):
-        value = utf8(b"1234")
-        self.failUnlessEqual(value, b"1234")
-        self.failUnless(isinstance(value, bytes))
+    def __delitem__(self, *args):
+        return self.__d.__delitem__(*args)
 
-    def test_bad_str(self):
-        value = utf8(b"\xab\xde")
-        # Two '?' symbols.
-        self.failUnlessEqual(value, b"\xef\xbf\xbd\xef\xbf\xbd")
-        self.failUnless(isinstance(value, bytes))
-
-    def test_low_unicode(self):
-        value = utf8(u"1234")
-        self.failUnlessEqual(value, b"1234")
-        self.failUnless(isinstance(value, bytes))
-
-    def test_high_unicode(self):
-        value = utf8(u"\u1234")
-        self.failUnlessEqual(value, b'\xe1\x88\xb4')
-        self.failUnless(isinstance(value, bytes))
-
-    def test_invalid(self):
-        self.failUnlessRaises(TypeError, utf8, 1234)
-
-add(Tutf8)
 
 class TDictMixin(TestCase):
 
@@ -136,6 +121,7 @@ class TDictMixin(TestCase):
 
 add(TDictMixin)
 
+
 class Tcdata(TestCase):
 
     ZERO = b"\x00\x00\x00\x00"
@@ -153,7 +139,7 @@ class Tcdata(TestCase):
         self.failUnlessEqual(cdata.uint_le(self.ZERO), 0)
         self.failUnlessEqual(cdata.uint_le(self.LEONE), 1)
         self.failUnlessEqual(cdata.uint_le(self.BEONE), 16777216)
-        self.failUnlessEqual(cdata.uint_le(self.NEGONE), 2**32-1)
+        self.failUnlessEqual(cdata.uint_le(self.NEGONE), 2 ** 32 - 1)
 
     def test_longlong_le(self):
         self.failUnlessEqual(cdata.longlong_le(self.ZERO * 2), 0)
@@ -163,7 +149,7 @@ class Tcdata(TestCase):
     def test_ulonglong_le(self):
         self.failUnlessEqual(cdata.ulonglong_le(self.ZERO * 2), 0)
         self.failUnlessEqual(cdata.ulonglong_le(self.LEONE + self.ZERO), 1)
-        self.failUnlessEqual(cdata.ulonglong_le(self.NEGONE * 2), 2**64-1)
+        self.failUnlessEqual(cdata.ulonglong_le(self.NEGONE * 2), 2 ** 64 - 1)
 
     def test_invalid_lengths(self):
         self.failUnlessRaises(cdata.error, cdata.int_le, b"")
@@ -188,6 +174,7 @@ class Tcdata(TestCase):
 
 add(Tcdata)
 
+
 class FileHandling(TestCase):
     def file(self, contents):
         import tempfile
@@ -288,26 +275,26 @@ class FileHandling(TestCase):
     def test_insert_6106_79_51760(self):
         # This appears to be due to ANSI C limitations in read/write on rb+
         # files. The problematic behavior only showed up in our mmap fallback
-        # code for transfers of this or similar sizes. 
-        data = u''.join(map(text_type, range(12574))) # 51760 bytes
+        # code for transfers of this or similar sizes.
+        data = u''.join(map(text_type, range(12574)))  # 51760 bytes
         data = data.encode("ascii")
         o = self.file(data)
         insert_bytes(o, 6106, 79)
-        self.failUnless(data[:6106+79] + data[79:] == self.read(o))
+        self.failUnless(data[:6106 + 79] + data[79:] == self.read(o))
 
     def test_delete_6106_79_51760(self):
         # This appears to be due to ANSI C limitations in read/write on rb+
         # files. The problematic behavior only showed up in our mmap fallback
-        # code for transfers of this or similar sizes. 
-        data = u''.join(map(text_type, range(12574))) # 51760 bytes
+        # code for transfers of this or similar sizes.
+        data = u''.join(map(text_type, range(12574)))  # 51760 bytes
         data = data.encode("ascii")
-        o = self.file(data[:6106+79] + data[79:])
+        o = self.file(data[:6106 + 79] + data[79:])
         delete_bytes(o, 6106, 79)
         self.failUnless(data == self.read(o))
 
     # Generate a bunch of random insertions, apply them, delete them,
     # and make sure everything is still correct.
-    # 
+    #
     # The num_runs and num_changes values are tuned to take about 10s
     # on my laptop, or about 30 seconds since we we have 3 variations
     # on insert/delete_bytes brokenness. If I ever get a faster
@@ -327,14 +314,16 @@ class FileHandling(TestCase):
             # Generate the list of changes to apply
             changes = []
             for i in range(num_changes):
-                change_size = random.randrange(min_change_size, max_change_size)
+                change_size = random.randrange(
+                    min_change_size, max_change_size)
                 change_offset = random.randrange(0, filesize)
                 filesize += change_size
                 changes.append((change_offset, change_size))
 
             # Apply the changes, and make sure they all took.
             for offset, size in changes:
-                buffer_size = random.randrange(min_buffer_size, max_buffer_size)
+                buffer_size = random.randrange(
+                    min_buffer_size, max_buffer_size)
                 insert_bytes(fobj, size, offset, BUFFER_SIZE=buffer_size)
             fobj.seek(0)
             self.failIfEqual(fobj.read(len(data)), data)
@@ -344,7 +333,8 @@ class FileHandling(TestCase):
             # Then, undo them.
             changes.reverse()
             for offset, size in changes:
-                buffer_size = random.randrange(min_buffer_size, max_buffer_size)
+                buffer_size = random.randrange(
+                    min_buffer_size, max_buffer_size)
                 delete_bytes(fobj, size, offset, BUFFER_SIZE=buffer_size)
             fobj.seek(0)
             self.failUnless(fobj.read() == data)
@@ -352,6 +342,95 @@ class FileHandling(TestCase):
 add(FileHandling)
 
 
+class FileHandlingMockedLock(FileHandling):
+
+    def setUp(self):
+        def MockLockF(*args, **kwargs):
+            raise IOError
+        self._orig_lockf = fcntl.lockf
+        fcntl.lockf = MockLockF
+
+    def tearDown(self):
+        fcntl.lockf = self._orig_lockf
+
+if fcntl:
+    add(FileHandlingMockedLock)
+
+
+class FileHandlingMockedMMapMove(FileHandling):
+
+    def setUp(self):
+        class MockMMap(object):
+            def __init__(self, *args, **kwargs):
+                pass
+
+            def move(self, dest, src, count):
+                raise ValueError
+
+            def close(self):
+                pass
+
+        self._orig_mmap = mmap.mmap
+        mmap.mmap = MockMMap
+
+    def tearDown(self):
+        mmap.mmap = self._orig_mmap
+
+add(FileHandlingMockedMMapMove)
+
+
+class FileHandlingMockedMMap(FileHandling):
+
+    def setUp(self):
+        def MockMMap2(*args, **kwargs):
+            raise EnvironmentError
+
+        self._orig_mmap = mmap.mmap
+        mmap.mmap = MockMMap2
+
+    def tearDown(self):
+        mmap.mmap = self._orig_mmap
+
+add(FileHandlingMockedMMap)
+
+
+class Tdict_match(TestCase):
+
+    def test_match(self):
+        self.assertEqual(dict_match({"*": 1}, "a"), 1)
+        self.assertEqual(dict_match({"*": 1}, "*"), 1)
+        self.assertEqual(dict_match({"*a": 1}, "ba"), 1)
+        self.assertEqual(dict_match({"?": 1}, "b"), 1)
+        self.assertEqual(dict_match({"[ab]": 1}, "b"), 1)
+
+    def test_nomatch(self):
+        self.assertEqual(dict_match({"*a": 1}, "ab"), None)
+        self.assertEqual(dict_match({"??": 1}, "a"), None)
+        self.assertEqual(dict_match({"[ab]": 1}, "c"), None)
+        self.assertEqual(dict_match({"[ab]": 1}, "[ab]"), None)
+
+add(Tdict_match)
+
+
+class Tenum(TestCase):
+
+    def test_enum(self):
+        @enum
+        class Foo(object):
+            FOO = 1
+            BAR = 3
+
+        self.assertEqual(Foo.FOO, 1)
+        self.assertTrue(isinstance(Foo.FOO, Foo))
+        self.assertEqual(repr(Foo.FOO), "Foo.FOO")
+        self.assertEqual(repr(Foo(3)), "Foo.BAR")
+        self.assertEqual(repr(Foo(42)), "Foo(42)")
+        self.assertEqual(str(Foo(42)), "42")
+        self.assertEqual(str(Foo(1)), "1")
+
+add(Tenum)
+
+
 class Tdecode_terminated(TestCase):
 
     def test_all(self):
@@ -391,3 +470,49 @@ class Tdecode_terminated(TestCase):
             truncated, "utf-8", strict=False)
 
 add(Tdecode_terminated)
+
+
+class Tsplit_escape(TestCase):
+    def test_split_escape(self):
+        inout = [
+            (("", ":"), [""]),
+            ((":", ":"), ["", ""]),
+            ((":", ":", 0), [":"]),
+            ((":b:c:", ":", 0), [":b:c:"]),
+            ((":b:c:", ":", 1), ["", "b:c:"]),
+            ((":b:c:", ":", 2), ["", "b", "c:"]),
+            ((":b:c:", ":", 3), ["", "b", "c", ""]),
+            (("a\\:b:c", ":"), ["a:b", "c"]),
+            (("a\\\\:b:c", ":"), ["a\\", "b", "c"]),
+            (("a\\\\\\:b:c\\:", ":"), ["a\\:b", "c:"]),
+            (("\\", ":"), [""]),
+            (("\\\\", ":"), ["\\"]),
+            (("\\\\a\\b", ":"), ["\\a\\b"]),
+        ]
+
+        for inargs, out in inout:
+            self.assertEqual(split_escape(*inargs), out)
+
+    def test_types(self):
+        parts = split_escape(b"\xff:\xff", b":")
+        self.assertEqual(parts, [b"\xff", b"\xff"])
+        self.assertTrue(isinstance(parts[0], bytes))
+
+        parts = split_escape(b"", b":")
+        self.assertEqual(parts, [b""])
+        self.assertTrue(isinstance(parts[0], bytes))
+
+        parts = split_escape(u"a:b", u":")
+        self.assertEqual(parts, [u"a", u"b"])
+        self.assertTrue(all(isinstance(p, text_type) for p in parts))
+
+        parts = split_escape(u"", u":")
+        self.assertEqual(parts, [u""])
+        self.assertTrue(all(isinstance(p, text_type) for p in parts))
+
+        parts = split_escape(u":", u":")
+        self.assertEqual(parts, [u"", u""])
+        self.assertTrue(all(isinstance(p, text_type) for p in parts))
+
+
+add(Tsplit_escape)
