@@ -2,10 +2,12 @@ import os
 import shutil
 import pickle
 from tests import add, TestCase
-from mutagen.id3 import ID3FileType
+from mutagen.id3 import ID3FileType, ID3
 from mutagen.easyid3 import EasyID3, error as ID3Error
+from mutagen._compat import PY3
 from tempfile import mkstemp
 
+
 class TEasyID3(TestCase):
 
     def setUp(self):
@@ -24,6 +26,11 @@ class TEasyID3(TestCase):
         mp3.pprint()
         self.failUnless(isinstance(mp3.tags, EasyID3))
 
+    def test_ignore_23(self):
+        self.id3["date"] = "2004"
+        self.id3.save(self.filename, v2_version=3)
+        self.assertEqual(ID3(self.filename).version, (2, 4, 0))
+
     def test_delete(self):
         self.id3["artist"] = "foobar"
         self.id3.save(self.filename)
@@ -36,8 +43,12 @@ class TEasyID3(TestCase):
         self.id3["artist"] = "baz"
         self.id3.pprint()
 
-    def test_has_key(self):
-        self.failIf(self.id3.has_key("foo"))
+    def test_in(self):
+        self.failIf("foo" in self.id3)
+
+    if not PY3:
+        def test_has_key(self):
+            self.failIf(self.id3.has_key("foo"))
 
     def test_empty_file(self):
         empty = os.path.join('tests', 'data', 'emptyfile.mp3')
@@ -82,13 +93,15 @@ class TEasyID3(TestCase):
             self.id3[key] = ["a test", "value"]
             self.id3.save(self.filename)
             id3 = EasyID3(self.filename)
-            self.failUnlessEqual(id3.get(key), ["a test", "value"])
+            # some keys end up in multiple frames and ID3.getall returns
+            # them in undefined order
+            self.failUnlessEqual(sorted(id3.get(key)), ["a test", "value"])
             self.failUnlessEqual(id3.keys(), [key])
 
             self.id3[key] = ["a test", "value"]
             self.id3.save(self.filename)
             id3 = EasyID3(self.filename)
-            self.failUnlessEqual(id3.get(key), ["a test", "value"])
+            self.failUnlessEqual(sorted(id3.get(key)), ["a test", "value"])
             self.failUnlessEqual(id3.keys(), [key])
 
             del(self.id3[key])
@@ -109,7 +122,7 @@ class TEasyID3(TestCase):
         self.failUnlessEqual(self.id3["date"], ["2004"])
         del(self.id3["date"])
         self.failIf("date" in self.id3.keys())
-        
+
     def test_write_date_double(self):
         self.id3["date"] = ["2004", "2005"]
         self.id3.save(self.filename)
@@ -192,31 +205,33 @@ class TEasyID3(TestCase):
     def test_gain_bad_key(self):
         self.failIf("replaygain_foo_gain" in self.id3)
         self.failIf(self.id3._EasyID3__id3.getall("RVA2"))
-        
+
     def test_gain_bad_value(self):
         self.failUnlessRaises(
             ValueError, self.id3.__setitem__, "replaygain_foo_gain", [])
         self.failUnlessRaises(
             ValueError, self.id3.__setitem__, "replaygain_foo_gain", ["foo"])
         self.failUnlessRaises(
-            ValueError, self.id3.__setitem__, "replaygain_foo_gain", ["1", "2"])
+            ValueError,
+            self.id3.__setitem__, "replaygain_foo_gain", ["1", "2"])
         self.failIf(self.id3._EasyID3__id3.getall("RVA2"))
-        
+
     def test_peak_bad_key(self):
         self.failIf("replaygain_foo_peak" in self.id3)
         self.failIf(self.id3._EasyID3__id3.getall("RVA2"))
-        
+
     def test_peak_bad_value(self):
         self.failUnlessRaises(
             ValueError, self.id3.__setitem__, "replaygain_foo_peak", [])
         self.failUnlessRaises(
             ValueError, self.id3.__setitem__, "replaygain_foo_peak", ["foo"])
         self.failUnlessRaises(
-            ValueError, self.id3.__setitem__, "replaygain_foo_peak", ["1", "1"])
+            ValueError,
+            self.id3.__setitem__, "replaygain_foo_peak", ["1", "1"])
         self.failUnlessRaises(
             ValueError, self.id3.__setitem__, "replaygain_foo_peak", ["3"])
         self.failIf(self.id3._EasyID3__id3.getall("RVA2"))
-        
+
     def test_gain_peak_get(self):
         self.id3["replaygain_foo_gain"] = "+3.5 dB"
         self.id3["replaygain_bar_peak"] = "0.5"
