@@ -2,14 +2,15 @@ import os
 import shutil
 import struct
 
-from mutagen._compat import cBytesIO
+from mutagen._compat import cBytesIO, PY3
 from tempfile import mkstemp
 from tests import TestCase, add
-from mutagen.mp4 import MP4, Atom, Atoms, MP4Tags, MP4Info, \
-     delete, MP4Cover, MP4MetadataError, MP4FreeForm, error
+from mutagen.mp4 import (MP4, Atom, Atoms, MP4Tags, MP4Info, delete, MP4Cover,
+                         MP4MetadataError, MP4FreeForm, error, AtomDataType,
+                         MP4MetadataValueError)
 from mutagen._util import cdata
-try: from os.path import devnull
-except ImportError: devnull = "/dev/null"
+from os import devnull
+
 
 class TAtom(TestCase):
 
@@ -37,7 +38,8 @@ class TAtom(TestCase):
             def __len__(self):
                 return 1 << 32
         data = TooBig(b"test")
-        try: len(data)
+        try:
+            len(data)
         except OverflowError:
             # Py_ssize_t is still only 32 bits on this system.
             self.failUnlessRaises(OverflowError, Atom.render, b"data", data)
@@ -65,6 +67,7 @@ class TAtom(TestCase):
 
 add(TAtom)
 
+
 class TAtoms(TestCase):
     filename = os.path.join("tests", "data", "has-tags.m4a")
 
@@ -98,6 +101,7 @@ class TAtoms(TestCase):
         repr(self.atoms)
 add(TAtoms)
 
+
 class TMP4Info(TestCase):
 
     def test_no_soun(self):
@@ -106,7 +110,7 @@ class TMP4Info(TestCase):
 
     def test_mdhd_version_1(self, soun=b"soun"):
         mdhd = Atom.render(b"mdhd", (b"\x01\x00\x00\x00" + b"\x00" * 16 +
-                                     b"\x00\x00\x00\x02" + # 2 Hz
+                                     b"\x00\x00\x00\x02" +  # 2 Hz
                                      b"\x00\x00\x00\x00\x00\x00\x00\x10"))
         hdlr = Atom.render(b"hdlr", b"\x00" * 8 + soun)
         mdia = Atom.render(b"mdia", mdhd + hdlr)
@@ -122,7 +126,7 @@ class TMP4Info(TestCase):
         mdia = Atom.render(b"mdia", hdlr)
         trak1 = Atom.render(b"trak", mdia)
         mdhd = Atom.render(b"mdhd", (b"\x01\x00\x00\x00" + b"\x00" * 16 +
-                                     b"\x00\x00\x00\x02" + # 2 Hz
+                                     b"\x00\x00\x00\x02" +  # 2 Hz
                                      b"\x00\x00\x00\x00\x00\x00\x00\x10"))
         hdlr = Atom.render(b"hdlr", b"\x00" * 8 + b"soun")
         mdia = Atom.render(b"mdia", mdhd + hdlr)
@@ -134,6 +138,7 @@ class TMP4Info(TestCase):
         self.failUnlessEqual(info.length, 8)
 add(TMP4Info)
 
+
 class TMP4Tags(TestCase):
 
     def wrap_ilst(self, data):
@@ -143,25 +148,62 @@ class TMP4Tags(TestCase):
         fileobj = cBytesIO(data)
         return MP4Tags(Atoms(fileobj), fileobj)
 
+    def test_parse_multiple_atoms(self):
+        # while we don't write multiple values as multiple atoms
+        # still read them
+        # https://bitbucket.org/lazka/mutagen/issue/165
+        data = Atom.render(b"data", b"\x00\x00\x00\x01" + b"\x00" * 4 + b"foo")
+        grp1 = Atom.render(b"\xa9grp", data)
+        data = Atom.render(b"data", b"\x00\x00\x00\x01" + b"\x00" * 4 + b"bar")
+        grp2 = Atom.render(b"\xa9grp", data)
+        tags = self.wrap_ilst(grp1 + grp2)
+        self.assertEqual(tags["\xa9grp"], [u"foo", u"bar"])
+
+    def test_purl(self):
+        # purl can have 0 or 1 flags (implicit or utf8)
+        data = Atom.render(b"data", b"\x00\x00\x00\x01" + b"\x00" * 4 + b"foo")
+        purl = Atom.render(b"purl", data)
+        tags = self.wrap_ilst(purl)
+        self.failUnlessEqual(tags["purl"], ["foo"])
+
+        data = Atom.render(b"data", b"\x00\x00\x00\x00" + b"\x00" * 4 + b"foo")
+        purl = Atom.render(b"purl", data)
+        tags = self.wrap_ilst(purl)
+        self.failUnlessEqual(tags["purl"], ["foo"])
+
+        # invalid flag
+        data = Atom.render(b"data", b"\x00\x00\x00\x03" + b"\x00" * 4 + b"foo")
+        purl = Atom.render(b"purl", data)
+        tags = self.wrap_ilst(purl)
+        self.assertFalse("purl" in tags)
+
+        self.assertTrue("purl" in tags._failed_atoms)
+
+        # invalid utf8
+        data = Atom.render(
+            b"data", b"\x00\x00\x00\x01" + b"\x00" * 4 + b"\xff")
+        purl = Atom.render(b"purl", data)
+        tags = self.wrap_ilst(purl)
+        self.assertFalse("purl" in tags)
+
     def test_genre(self):
         data = Atom.render(b"data", b"\x00" * 8 + b"\x00\x01")
         genre = Atom.render(b"gnre", data)
         tags = self.wrap_ilst(genre)
-        self.failIf(b"gnre" in tags)
-        self.failUnlessEqual(tags[b"\xa9gen"], ["Blues"])
+        self.failIf("gnre" in tags)
+        self.failUnlessEqual(tags["\xa9gen"], ["Blues"])
 
     def test_empty_cpil(self):
         cpil = Atom.render(b"cpil", Atom.render(b"data", b"\x00" * 8))
         tags = self.wrap_ilst(cpil)
-        self.failUnless(b"cpil" in tags)
-        self.failIf(tags[b"cpil"])
+        self.assertFalse("cpil" in tags)
 
     def test_genre_too_big(self):
         data = Atom.render(b"data", b"\x00" * 8 + b"\x01\x00")
         genre = Atom.render(b"gnre", data)
         tags = self.wrap_ilst(genre)
-        self.failIf(b"gnre" in tags)
-        self.failIf(b"\xa9gen" in tags)
+        self.failIf("gnre" in tags)
+        self.failIf("\xa9gen" in tags)
 
     def test_strips_unknown_types(self):
         data = Atom.render(b"data", b"\x00" * 8 + b"whee")
@@ -176,80 +218,175 @@ class TMP4Tags(TestCase):
         self.failIf(tags)
 
     def test_bad_covr(self):
-        data = Atom.render(b"foob", b"\x00\x00\x00\x0E" + b"\x00" * 4 + b"whee")
-        covr = Atom.render(b"covr", data)
-        self.failUnlessRaises(MP4MetadataError, self.wrap_ilst, covr)
-
-    def test_covr_blank_format(self):
-        data = Atom.render(b"data", b"\x00\x00\x00\x00" + b"\x00" * 4 + b"whee")
+        data = Atom.render(
+            b"foob", b"\x00\x00\x00\x0E" + b"\x00" * 4 + b"whee")
         covr = Atom.render(b"covr", data)
         tags = self.wrap_ilst(covr)
-        self.failUnlessEqual(MP4Cover.FORMAT_JPEG, tags[b"covr"][0].imageformat)
+        self.assertFalse(tags)
+
+    def test_covr_blank_format(self):
+        data = Atom.render(
+            b"data", b"\x00\x00\x00\x00" + b"\x00" * 4 + b"whee")
+        covr = Atom.render(b"covr", data)
+        tags = self.wrap_ilst(covr)
+        self.failUnlessEqual(
+            MP4Cover.FORMAT_JPEG, tags["covr"][0].imageformat)
 
     def test_render_bool(self):
-        self.failUnlessEqual(MP4Tags()._MP4Tags__render_bool(b'pgap', True),
-                             b"\x00\x00\x00\x19pgap\x00\x00\x00\x11data"
-                             b"\x00\x00\x00\x15\x00\x00\x00\x00\x01")
-        self.failUnlessEqual(MP4Tags()._MP4Tags__render_bool(b'pgap', False),
-                             b"\x00\x00\x00\x19pgap\x00\x00\x00\x11data"
-                             b"\x00\x00\x00\x15\x00\x00\x00\x00\x00")
+        self.failUnlessEqual(
+            MP4Tags()._MP4Tags__render_bool('pgap', True),
+            b"\x00\x00\x00\x19pgap\x00\x00\x00\x11data"
+            b"\x00\x00\x00\x15\x00\x00\x00\x00\x01"
+        )
+        self.failUnlessEqual(
+            MP4Tags()._MP4Tags__render_bool('pgap', False),
+            b"\x00\x00\x00\x19pgap\x00\x00\x00\x11data"
+            b"\x00\x00\x00\x15\x00\x00\x00\x00\x00"
+        )
 
     def test_render_text(self):
         self.failUnlessEqual(
-             MP4Tags()._MP4Tags__render_text(b'purl', ['http://foo/bar.xml'], 0),
-             b"\x00\x00\x00*purl\x00\x00\x00\"data\x00\x00\x00\x00\x00\x00"
-             b"\x00\x00http://foo/bar.xml")
+            MP4Tags()._MP4Tags__render_text(
+                'purl', ['http://foo/bar.xml'], 0),
+            b"\x00\x00\x00*purl\x00\x00\x00\"data\x00\x00\x00\x00\x00\x00"
+            b"\x00\x00http://foo/bar.xml"
+        )
         self.failUnlessEqual(
-             MP4Tags()._MP4Tags__render_text(b'aART', [u'\u0041lbum Artist']),
-             b"\x00\x00\x00$aART\x00\x00\x00\x1cdata\x00\x00\x00\x01\x00\x00"
-             b"\x00\x00\x41lbum Artist")
+            MP4Tags()._MP4Tags__render_text(
+                'aART', [u'\u0041lbum Artist']),
+            b"\x00\x00\x00$aART\x00\x00\x00\x1cdata\x00\x00\x00\x01\x00\x00"
+            b"\x00\x00\x41lbum Artist"
+        )
         self.failUnlessEqual(
-             MP4Tags()._MP4Tags__render_text(b'aART', [u'Album Artist', u'Whee']),
-             b"\x00\x00\x008aART\x00\x00\x00\x1cdata\x00\x00\x00\x01\x00\x00"
-             b"\x00\x00Album Artist\x00\x00\x00\x14data\x00\x00\x00\x01\x00"
-             b"\x00\x00\x00Whee")
-        
+            MP4Tags()._MP4Tags__render_text(
+                'aART', [u'Album Artist', u'Whee']),
+            b"\x00\x00\x008aART\x00\x00\x00\x1cdata\x00\x00\x00\x01\x00\x00"
+            b"\x00\x00Album Artist\x00\x00\x00\x14data\x00\x00\x00\x01\x00"
+            b"\x00\x00\x00Whee"
+        )
+
     def test_render_data(self):
         self.failUnlessEqual(
-             MP4Tags()._MP4Tags__render_data(b'aART', 1, [b'whee']),
-             b"\x00\x00\x00\x1caART"
-             b"\x00\x00\x00\x14data\x00\x00\x00\x01\x00\x00\x00\x00whee")
+            MP4Tags()._MP4Tags__render_data('aART', 0, 1, [b'whee']),
+            b"\x00\x00\x00\x1caART"
+            b"\x00\x00\x00\x14data\x00\x00\x00\x01\x00\x00\x00\x00whee"
+        )
         self.failUnlessEqual(
-             MP4Tags()._MP4Tags__render_data(b'aART', 2, [b'whee', b'wee']),
-             b"\x00\x00\x00/aART"
-             b"\x00\x00\x00\x14data\x00\x00\x00\x02\x00\x00\x00\x00whee"
-             b"\x00\x00\x00\x13data\x00\x00\x00\x02\x00\x00\x00\x00wee")
+            MP4Tags()._MP4Tags__render_data('aART', 0, 2, [b'whee', b'wee']),
+            b"\x00\x00\x00/aART"
+            b"\x00\x00\x00\x14data\x00\x00\x00\x02\x00\x00\x00\x00whee"
+            b"\x00\x00\x00\x13data\x00\x00\x00\x02\x00\x00\x00\x00wee"
+        )
 
     def test_bad_text_data(self):
         data = Atom.render(b"datA", b"\x00\x00\x00\x01\x00\x00\x00\x00whee")
         data = Atom.render(b"aART", data)
-        self.failUnlessRaises(MP4MetadataError, self.wrap_ilst, data)
+        tags = self.wrap_ilst(data)
+        self.assertFalse(tags)
+
+    def test_bad_cprt(self):
+        data = Atom.render(b"cprt", b"\x00\x00\x00#data\x00")
+        tags = self.wrap_ilst(data)
+        self.assertFalse(tags)
+
+    def test_write_back_bad_atoms(self):
+        # write a broken atom and try to load it
+        data = Atom.render(b"datA", b"\x00\x00\x00\x01\x00\x00\x00\x00wheeee")
+        data = Atom.render(b"aART", data)
+        tags = self.wrap_ilst(data)
+        self.assertFalse(tags)
+
+        # save it into an existing mp4
+        original = os.path.join("tests", "data", "has-tags.m4a")
+        fd, filename = mkstemp(suffix='.mp4')
+        os.close(fd)
+        shutil.copy(original, filename)
+        delete(filename)
+
+        # it should still end up in the file
+        tags.save(filename)
+        with open(filename, "rb") as h:
+            self.assertTrue(b"wheeee" in h.read())
+
+        # if we define our own aART throw away the broken one
+        tags["aART"] = ["new"]
+        tags.save(filename)
+        with open(filename, "rb") as h:
+            self.assertFalse(b"wheeee" in h.read())
+
+        # add the broken one back and delete all tags including the broken one
+        del tags["aART"]
+        tags.save(filename)
+        with open(filename, "rb") as h:
+            self.assertTrue(b"wheeee" in h.read())
+        delete(filename)
+        with open(filename, "rb") as h:
+            self.assertFalse(b"wheeee" in h.read())
 
     def test_render_freeform(self):
+        data = (
+            b"\x00\x00\x00a----"
+            b"\x00\x00\x00\"mean\x00\x00\x00\x00net.sacredchao.Mutagen"
+            b"\x00\x00\x00\x10name\x00\x00\x00\x00test"
+            b"\x00\x00\x00\x14data\x00\x00\x00\x01\x00\x00\x00\x00whee"
+            b"\x00\x00\x00\x13data\x00\x00\x00\x01\x00\x00\x00\x00wee"
+        )
+
+        key = '----:net.sacredchao.Mutagen:test'
         self.failUnlessEqual(
-             MP4Tags()._MP4Tags__render_freeform(
-             b'----:net.sacredchao.Mutagen:test', [b'whee', b'wee']),
-             b"\x00\x00\x00a----"
-             b"\x00\x00\x00\"mean\x00\x00\x00\x00net.sacredchao.Mutagen"
-             b"\x00\x00\x00\x10name\x00\x00\x00\x00test"
-             b"\x00\x00\x00\x14data\x00\x00\x00\x01\x00\x00\x00\x00whee"
-             b"\x00\x00\x00\x13data\x00\x00\x00\x01\x00\x00\x00\x00wee")
+            MP4Tags()._MP4Tags__render_freeform(key, [b'whee', b'wee']), data)
+
+    def test_parse_freeform(self):
+        double_data = (
+            b"\x00\x00\x00a----"
+            b"\x00\x00\x00\"mean\x00\x00\x00\x00net.sacredchao.Mutagen"
+            b"\x00\x00\x00\x10name\x00\x00\x00\x00test"
+            b"\x00\x00\x00\x14data\x00\x00\x00\x01\x00\x00\x00\x00whee"
+            b"\x00\x00\x00\x13data\x00\x00\x00\x01\x00\x00\x00\x00wee"
+        )
+
+        key = '----:net.sacredchao.Mutagen:test'
+        double_atom = \
+            MP4Tags()._MP4Tags__render_freeform(key, [b'whee', b'wee'])
+
+        tags = self.wrap_ilst(double_data)
+        self.assertTrue(key in tags)
+        self.assertEqual(tags[key], [b'whee', b'wee'])
+
+        tags2 = self.wrap_ilst(double_atom)
+        self.assertEqual(tags, tags2)
+
+    def test_multi_freeform(self):
+        # merge multiple freeform tags with the same key
+        mean = Atom.render(b"mean", b"\x00" * 4 + b"net.sacredchao.Mutagen")
+        name = Atom.render(b"name", b"\x00" * 4 + b"foo")
+
+        data = Atom.render(b"data", b"\x00\x00\x00\x01" + b"\x00" * 4 + b"bar")
+        result = Atom.render(b"----", mean + name + data)
+        data = Atom.render(
+            b"data", b"\x00\x00\x00\x01" + b"\x00" * 4 + b"quux")
+        result += Atom.render(b"----", mean + name + data)
+        tags = self.wrap_ilst(result)
+        values = tags["----:net.sacredchao.Mutagen:foo"]
+        self.assertEqual(values[0], b"bar")
+        self.assertEqual(values[1], b"quux")
 
     def test_bad_freeform(self):
         mean = Atom.render(b"mean", b"net.sacredchao.Mutagen")
         name = Atom.render(b"name", b"empty test key")
         bad_freeform = Atom.render(b"----", b"\x00" * 4 + mean + name)
-        self.failUnlessRaises(MP4MetadataError, self.wrap_ilst, bad_freeform)
+        tags = self.wrap_ilst(bad_freeform)
+        self.assertFalse(tags)
 
     def test_pprint_non_text_list(self):
         tags = MP4Tags()
-        tags[b"tmpo"] = [120, 121]
-        tags[b"trck"] = [(1, 2), (3, 4)]
+        tags["tmpo"] = [120, 121]
+        tags["trck"] = [(1, 2), (3, 4)]
         tags.pprint()
 
     def test_freeform_data(self):
         # http://code.google.com/p/mutagen/issues/detail?id=103
-        key = b"----:com.apple.iTunes:Encoding Params"
+        key = "----:com.apple.iTunes:Encoding Params"
         value = (b"vers\x00\x00\x00\x01acbf\x00\x00\x00\x01brat\x00\x01\xf4"
                  b"\x00cdcv\x00\x01\x05\x04")
 
@@ -261,18 +398,19 @@ class TMP4Tags(TestCase):
         tags = self.wrap_ilst(Atom.render(b"----", data))
         v = tags[key][0]
         self.failUnlessEqual(v, value)
-        self.failUnlessEqual(v.dataformat, MP4FreeForm.FORMAT_DATA)
+        self.failUnlessEqual(v.dataformat, AtomDataType.IMPLICIT)
 
         data = MP4Tags()._MP4Tags__render_freeform(key, v)
         v = self.wrap_ilst(data)[key][0]
-        self.failUnlessEqual(v.dataformat, MP4FreeForm.FORMAT_DATA)
+        self.failUnlessEqual(v.dataformat, AtomDataType.IMPLICIT)
 
         data = MP4Tags()._MP4Tags__render_freeform(key, value)
         v = self.wrap_ilst(data)[key][0]
-        self.failUnlessEqual(v.dataformat, MP4FreeForm.FORMAT_TEXT)
+        self.failUnlessEqual(v.dataformat, AtomDataType.UTF8)
 
 add(TMP4Tags)
 
+
 class TMP4(TestCase):
     def setUp(self):
         fd, self.filename = mkstemp(suffix='.m4a')
@@ -281,9 +419,10 @@ class TMP4(TestCase):
         self.audio = MP4(self.filename)
 
     def faad(self):
-        if not have_faad: return
-        value = os.system("faad %s -o %s > %s 2> %s" % (
-                self.filename, devnull, devnull, devnull))
+        if not have_faad:
+            return
+        value = os.system("faad %s -o %s > %s 2> %s" % (self.filename, devnull,
+                                                        devnull, devnull))
         self.failIf(value and value != NOTFOUND)
 
     def test_score(self):
@@ -308,24 +447,24 @@ class TMP4(TestCase):
         self.failUnlessAlmostEqual(3.7, self.audio.info.length, 1)
 
     def test_padding(self):
-        self.audio[b"\xa9nam"] = u"wheeee" * 10
+        self.audio["\xa9nam"] = u"wheeee" * 10
         self.audio.save()
         size1 = os.path.getsize(self.audio.filename)
-        self.audio[b"\xa9nam"] = u"wheeee" * 11
+        self.audio["\xa9nam"] = u"wheeee" * 11
         self.audio.save()
         size2 = os.path.getsize(self.audio.filename)
         self.failUnless(size1, size2)
 
     def test_padding_2(self):
-        self.audio[b"\xa9nam"] = u"wheeee" * 10
+        self.audio["\xa9nam"] = u"wheeee" * 10
         self.audio.save()
         # Reorder "free" and "ilst" atoms
         fileobj = open(self.audio.filename, "rb+")
         atoms = Atoms(fileobj)
         meta = atoms[b"moov", b"udta", b"meta"]
         meta_length1 = meta.length
-        ilst = meta[b"ilst",]
-        free = meta[b"free",]
+        ilst = meta[b"ilst", ]
+        free = meta[b"free", ]
         self.failUnlessEqual(ilst.offset + ilst.length, free.offset)
         fileobj.seek(ilst.offset)
         ilst_data = fileobj.read(ilst.length)
@@ -337,20 +476,20 @@ class TMP4(TestCase):
         fileobj = open(self.audio.filename, "rb+")
         atoms = Atoms(fileobj)
         meta = atoms[b"moov", b"udta", b"meta"]
-        ilst = meta[b"ilst",]
-        free = meta[b"free",]
+        ilst = meta[b"ilst", ]
+        free = meta[b"free", ]
         self.failUnlessEqual(free.offset + free.length, ilst.offset)
         fileobj.close()
         # Save the file
-        self.audio[b"\xa9nam"] = u"wheeee" * 11
+        self.audio["\xa9nam"] = u"wheeee" * 11
         self.audio.save()
         # Check the order of "free" and "ilst" atoms
         fileobj = open(self.audio.filename, "rb+")
         atoms = Atoms(fileobj)
         fileobj.close()
         meta = atoms[b"moov", b"udta", b"meta"]
-        ilst = meta[b"ilst",]
-        free = meta[b"free",]
+        ilst = meta[b"ilst", ]
+        free = meta[b"free", ]
         self.failUnlessEqual(meta.length, meta_length1)
         self.failUnlessEqual(ilst.offset + ilst.length, free.offset)
 
@@ -364,112 +503,147 @@ class TMP4(TestCase):
             self.faad()
 
     def test_unicode(self):
-        self.set_key(b'\xa9nam', [b'\xe3\x82\x8a\xe3\x81\x8b'],
-                     result=[u'\u308a\u304b'])
+        try:
+            self.set_key('\xa9nam', [b'\xe3\x82\x8a\xe3\x81\x8b'],
+                         result=[u'\u308a\u304b'])
+        except MP4MetadataValueError:
+            if not PY3:
+                raise
+
+    def test_preserve_freeform(self):
+        self.set_key('----:net.sacredchao.Mutagen:test key',
+                     [MP4FreeForm(b'woooo', 142, 42)])
+
+    def test_freeform_cmp(self):
+        self.assertReallyEqual(
+            MP4FreeForm(b'woooo', 142, 42), MP4FreeForm(b'woooo', 142, 42))
+        self.assertReallyNotEqual(
+            MP4FreeForm(b'woooo', 142, 43), MP4FreeForm(b'woooo', 142, 42))
+        self.assertReallyNotEqual(
+            MP4FreeForm(b'woooo', 143, 42), MP4FreeForm(b'woooo', 142, 42))
+        self.assertReallyNotEqual(
+            MP4FreeForm(b'wooox', 142, 42), MP4FreeForm(b'woooo', 142, 42))
+
+    def test_cover_cmp(self):
+        self.assertReallyEqual(
+            MP4Cover(b'woooo', 142), MP4Cover(b'woooo', 142))
+        self.assertReallyNotEqual(
+            MP4Cover(b'woooo', 143), MP4Cover(b'woooo', 142))
+        self.assertReallyNotEqual(
+            MP4Cover(b'woooo', 142), MP4Cover(b'wooox', 142))
+
+    def test_invalid_text(self):
+        self.assertRaises(
+            MP4MetadataValueError, self.set_key, '\xa9nam', [b'\xff'])
 
     def test_save_text(self):
-        self.set_key(b'\xa9nam', [u"Some test name"])
+        self.set_key('\xa9nam', [u"Some test name"])
 
     def test_save_texts(self):
-        self.set_key(b'\xa9nam', [u"Some test name", u"One more name"])
+        self.set_key('\xa9nam', [u"Some test name", u"One more name"])
 
     def test_freeform(self):
-        self.set_key(b'----:net.sacredchao.Mutagen:test key', [b"whee"])
+        self.set_key('----:net.sacredchao.Mutagen:test key', [b"whee"])
 
     def test_freeform_2(self):
-        self.set_key(b'----:net.sacredchao.Mutagen:test key', b"whee", [b"whee"])
+        self.set_key(
+            '----:net.sacredchao.Mutagen:test key', b"whee", [b"whee"])
 
     def test_freeforms(self):
-        self.set_key(b'----:net.sacredchao.Mutagen:test key', [b"whee", b"uhh"])
+        self.set_key(
+            '----:net.sacredchao.Mutagen:test key', [b"whee", b"uhh"])
 
     def test_freeform_bin(self):
-        self.set_key(b'----:net.sacredchao.Mutagen:test key', [
-            MP4FreeForm(b'woooo', MP4FreeForm.FORMAT_TEXT),
-            MP4FreeForm(b'hoooo', MP4FreeForm.FORMAT_DATA),
+        self.set_key('----:net.sacredchao.Mutagen:test key', [
+            MP4FreeForm(b'woooo', AtomDataType.UTF8),
+            MP4FreeForm(b'hoooo', AtomDataType.IMPLICIT),
             MP4FreeForm(b'boooo'),
         ])
 
     def test_tracknumber(self):
-        self.set_key(b'trkn', [(1, 10)])
-        self.set_key(b'trkn', [(1, 10), (5, 20)], faad=False)
-        self.set_key(b'trkn', [])
+        self.set_key('trkn', [(1, 10)])
+        self.set_key('trkn', [(1, 10), (5, 20)], faad=False)
+        self.set_key('trkn', [])
 
     def test_disk(self):
-        self.set_key(b'disk', [(18, 0)])
-        self.set_key(b'disk', [(1, 10), (5, 20)], faad=False)
-        self.set_key(b'disk', [])
+        self.set_key('disk', [(18, 0)])
+        self.set_key('disk', [(1, 10), (5, 20)], faad=False)
+        self.set_key('disk', [])
 
     def test_tracknumber_too_small(self):
-        self.failUnlessRaises(ValueError, self.set_key, b'trkn', [(-1, 0)])
-        self.failUnlessRaises(ValueError, self.set_key, b'trkn', [(2**18, 1)])
+        self.failUnlessRaises(ValueError, self.set_key, 'trkn', [(-1, 0)])
+        self.failUnlessRaises(
+            ValueError, self.set_key, 'trkn', [(2 ** 18, 1)])
 
     def test_disk_too_small(self):
-        self.failUnlessRaises(ValueError, self.set_key, b'disk', [(-1, 0)])
-        self.failUnlessRaises(ValueError, self.set_key, b'disk', [(2**18, 1)])
+        self.failUnlessRaises(ValueError, self.set_key, 'disk', [(-1, 0)])
+        self.failUnlessRaises(
+            ValueError, self.set_key, 'disk', [(2 ** 18, 1)])
 
     def test_tracknumber_wrong_size(self):
-        self.failUnlessRaises(ValueError, self.set_key, b'trkn', (1,))
-        self.failUnlessRaises(ValueError, self.set_key, b'trkn', (1, 2, 3,))
-        self.failUnlessRaises(ValueError, self.set_key, b'trkn', [(1,)])
-        self.failUnlessRaises(ValueError, self.set_key, b'trkn', [(1, 2, 3,)])
+        self.failUnlessRaises(ValueError, self.set_key, 'trkn', (1,))
+        self.failUnlessRaises(ValueError, self.set_key, 'trkn', (1, 2, 3,))
+        self.failUnlessRaises(ValueError, self.set_key, 'trkn', [(1,)])
+        self.failUnlessRaises(ValueError, self.set_key, 'trkn', [(1, 2, 3,)])
 
     def test_disk_wrong_size(self):
-        self.failUnlessRaises(ValueError, self.set_key, b'disk', [(1,)])
-        self.failUnlessRaises(ValueError, self.set_key, b'disk', [(1, 2, 3,)])
+        self.failUnlessRaises(ValueError, self.set_key, 'disk', [(1,)])
+        self.failUnlessRaises(ValueError, self.set_key, 'disk', [(1, 2, 3,)])
 
     def test_tempo(self):
-        self.set_key(b'tmpo', [150])
-        self.set_key(b'tmpo', [])
+        self.set_key('tmpo', [150])
+        self.set_key('tmpo', [])
 
     def test_tempos(self):
-        self.set_key(b'tmpo', [160, 200], faad=False)
+        self.set_key('tmpo', [160, 200], faad=False)
 
     def test_tempo_invalid(self):
         for badvalue in [[10000000], [-1], 10, "foo"]:
             self.failUnlessRaises(ValueError, self.set_key, 'tmpo', badvalue)
 
     def test_compilation(self):
-        self.set_key(b'cpil', True)
+        self.set_key('cpil', True)
 
     def test_compilation_false(self):
-        self.set_key(b'cpil', False)
+        self.set_key('cpil', False)
 
     def test_gapless(self):
-        self.set_key(b'pgap', True)
+        self.set_key('pgap', True)
 
     def test_gapless_false(self):
-        self.set_key(b'pgap', False)
+        self.set_key('pgap', False)
 
     def test_podcast(self):
-        self.set_key(b'pcst', True)
+        self.set_key('pcst', True)
 
     def test_podcast_false(self):
-        self.set_key(b'pcst', False)
+        self.set_key('pcst', False)
 
     def test_cover(self):
-        self.set_key(b'covr', [b'woooo'])
+        self.set_key('covr', [b'woooo'])
 
     def test_cover_png(self):
-        self.set_key(b'covr', [
+        self.set_key('covr', [
             MP4Cover(b'woooo', MP4Cover.FORMAT_PNG),
             MP4Cover(b'hoooo', MP4Cover.FORMAT_JPEG),
         ])
 
     def test_podcast_url(self):
-        self.set_key(b'purl', ['http://pdl.warnerbros.com/wbie/justiceleagueheroes/audio/JLH_EA.xml'])
+        self.set_key('purl', ['http://pdl.warnerbros.com/wbie/'
+                               'justiceleagueheroes/audio/JLH_EA.xml'])
 
     def test_episode_guid(self):
-        self.set_key(b'catg', ['falling-star-episode-1'])
+        self.set_key('catg', ['falling-star-episode-1'])
 
     def test_pprint(self):
         self.failUnless(self.audio.pprint())
 
     def test_pprint_binary(self):
-        self.audio[b"covr"] = "\x00\xa9\garbage"
+        self.audio["covr"] = [b"\x00\xa9\garbage"]
         self.failUnless(self.audio.pprint())
 
     def test_pprint_pair(self):
-        self.audio[b"cpil"] = (1, 10)
+        self.audio["cpil"] = (1, 10)
         self.failUnless("cpil=(1, 10)" in self.audio.pprint())
 
     def test_delete(self):
@@ -485,7 +659,7 @@ class TMP4(TestCase):
         self.faad()
 
     def test_reads_unknown_text(self):
-        self.set_key(b"foob", [u"A test"])
+        self.set_key("foob", [u"A test"])
 
     def __read_offsets(self, filename):
         fileobj = open(filename, 'rb')
@@ -523,7 +697,7 @@ class TMP4(TestCase):
 
     def test_update_offsets(self):
         aa = self.__read_offsets(self.original)
-        self.audio[b"\xa9nam"] = "wheeeeeeee"
+        self.audio["\xa9nam"] = "wheeeeeeee"
         self.audio.save()
         bb = self.__read_offsets(self.filename)
         for a, b in zip(aa, bb):
@@ -571,14 +745,15 @@ class TMP4Datatypes(TMP4HasTags):
     original = os.path.join("tests", "data", "has-tags.m4a")
 
     def test_has_freeform(self):
-        key = b"----:com.apple.iTunes:iTunNORM"
+        key = "----:com.apple.iTunes:iTunNORM"
         self.failUnless(key in self.audio.tags)
         ff = self.audio.tags[key]
-        self.failUnlessEqual(ff[0].dataformat, MP4FreeForm.FORMAT_TEXT)
+        self.failUnlessEqual(ff[0].dataformat, AtomDataType.UTF8)
+        self.failUnlessEqual(ff[0].version, 0)
 
     def test_has_covr(self):
-        self.failUnless(b'covr' in self.audio.tags)
-        covr = self.audio.tags[b'covr']
+        self.failUnless('covr' in self.audio.tags)
+        covr = self.audio.tags['covr']
         self.failUnlessEqual(len(covr), 2)
         self.failUnlessEqual(covr[0].imageformat, MP4Cover.FORMAT_PNG)
         self.failUnlessEqual(covr[1].imageformat, MP4Cover.FORMAT_JPEG)
@@ -591,14 +766,15 @@ class TMP4CovrWithName(TMP4):
     original = os.path.join("tests", "data", "covr-with-name.m4a")
 
     def test_has_covr(self):
-        self.failUnless(b'covr' in self.audio.tags)
-        covr = self.audio.tags[b'covr']
+        self.failUnless('covr' in self.audio.tags)
+        covr = self.audio.tags['covr']
         self.failUnlessEqual(len(covr), 2)
         self.failUnlessEqual(covr[0].imageformat, MP4Cover.FORMAT_PNG)
         self.failUnlessEqual(covr[1].imageformat, MP4Cover.FORMAT_JPEG)
 
 add(TMP4CovrWithName)
 
+
 class TMP4HasTags64Bit(TMP4HasTags):
     original = os.path.join("tests", "data", "truncated-64bit.mp4")
 
@@ -617,6 +793,7 @@ class TMP4HasTags64Bit(TMP4HasTags):
 
 add(TMP4HasTags64Bit)
 
+
 class TMP4NoTagsM4A(TMP4):
     original = os.path.join("tests", "data", "no-tags.m4a")
 
@@ -629,6 +806,7 @@ class TMP4NoTagsM4A(TMP4):
 
 add(TMP4NoTagsM4A)
 
+
 class TMP4NoTags3G2(TMP4):
     original = os.path.join("tests", "data", "no-tags.3g2")
 
@@ -646,6 +824,7 @@ class TMP4NoTags3G2(TMP4):
 
 add(TMP4NoTags3G2)
 
+
 class TMP4UpdateParents64Bit(TestCase):
     original = os.path.join("tests", "data", "64bit.mp4")
 
@@ -660,7 +839,7 @@ class TMP4UpdateParents64Bit(TestCase):
             self.assertEqual(77, atoms.atoms[0].length)
             self.assertEqual(61, atoms.atoms[0].children[0].length)
             tags = MP4Tags(atoms, fileobj)
-            tags[b'pgap'] = True
+            tags['pgap'] = True
             tags.save(self.filename)
 
         with open(self.filename, "rb") as fileobj:
